"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
/*
 * ng2-chrome-storage
 * @author Hasan Hameed <hasan.hameed07@gmail.com>
 * Url: https://github.com/hasanhameed07/ng2-chrome-storage.git
 */
var core_1 = require('@angular/core');
var Rx_1 = require('rxjs/Rx');
var settings_class_1 = require('./settings.class');
var ChromeStorage = (function () {
    function ChromeStorage(zone, _settings) {
        this.zone = zone;
        this.storeKey = ''; // chrome storage key
        var usethisSettings = (_settings) ? _settings : new settings_class_1.Settings();
        this.config = usethisSettings.data;
        this.storeKey = usethisSettings.storeKey;
    }
    // to be used inside a resolver
    ChromeStorage.prototype.load = function () {
        var _this = this;
        return this.getChrome(this.storeKey, this.config).then(function (data) {
            _this.config = data;
            return data;
        });
    };
    // save an object
    ChromeStorage.prototype.setAll = function (settings, key) {
        var _this = this;
        if (key === void 0) { key = this.storeKey; }
        return new Promise(function (resolve, reject) {
            if (chrome !== undefined && chrome.storage !== undefined) {
                var saveObj = {};
                saveObj[key] = settings;
                chrome.storage.sync.set(/* String or Array */ saveObj, function () { return _this.zone.run(function () {
                    resolve(true);
                }); });
            }
            else {
                // Put the object into storage
                localStorage.setItem(key, JSON.stringify(settings));
                // hack to resolve storage change event on the same window
                window.dispatchEvent(new Event('storage'));
                resolve(true);
            }
        });
    };
    // remove a key
    ChromeStorage.prototype.remove = function (key) {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (chrome !== undefined && chrome.storage !== undefined) {
                chrome.storage.sync.remove(/* String or Array */ key, function () { return _this.zone.run(function () {
                    resolve(true);
                }); });
            }
            else {
                localStorage.removeItem(key);
                resolve(true);
            }
        });
    };
    // clears the storage
    ChromeStorage.prototype.clear = function () {
        var _this = this;
        return new Promise(function (resolve, reject) {
            if (chrome !== undefined && chrome.storage !== undefined) {
                chrome.storage.sync.clear(function () { return _this.zone.run(function () {
                    resolve(true);
                }); });
            }
            else {
                localStorage.clear();
                resolve(true);
            }
        });
    };
    ChromeStorage.prototype.getChrome = function (key, defaults) {
        var _this = this;
        if (defaults === void 0) { defaults = {}; }
        return new Promise(function (resolve, reject) {
            if (chrome !== undefined && chrome.storage !== undefined) {
                var saveObj = {};
                saveObj[key] = defaults;
                chrome.storage.sync.get(/* String or Array */ saveObj, function (data) { return _this.zone.run(function () {
                    resolve(data[key]);
                }); });
            }
            else {
                var object = (localStorage.getItem(key) === null) ? defaults : JSON.parse(localStorage.getItem(key));
                resolve(object);
            }
        });
    };
    //  change detection
    ChromeStorage.prototype.onChange = function (key) {
        var _this = this;
        if (key === void 0) { key = this.storeKey; }
        return Rx_1.Observable.create(function (observer) {
            if (chrome !== undefined && chrome.storage !== undefined) {
                chrome.storage.onChanged.addListener(function (changes, namespace) { return _this.zone.run(function () {
                    if (!key) {
                        // give all changes
                        observer.next(changes);
                    }
                    else {
                        for (var tkey in changes) {
                            if (changes.hasOwnProperty(tkey)) {
                                var storageChange = changes[tkey];
                                if (tkey === key) {
                                    _this.config = storageChange.newValue;
                                    observer.next(storageChange.newValue);
                                }
                            }
                        }
                    }
                }); });
            }
            else {
                window.addEventListener('storage', function () { return _this.zone.run(function () {
                    var object = JSON.parse(localStorage.getItem(key));
                    _this.config = object;
                    observer.next(object);
                }); }, false);
            }
        });
    };
    ChromeStorage = __decorate([
        core_1.Injectable(),
        __param(1, core_1.Optional()), 
        __metadata('design:paramtypes', [core_1.NgZone, settings_class_1.Settings])
    ], ChromeStorage);
    return ChromeStorage;
}());
exports.ChromeStorage = ChromeStorage;
//# sourceMappingURL=ng2-chrome-storage.service.js.map